/* This is the header file that contains the functionality to generate the Sudoku.
To generate a Sudoku the user has to create two objects of class Sudoku and call
the generate function passing the objects as parameters and also the number of
numbers to hide. The program uses brute force method along with backtracking to
solve Sudoku and generates by solving a Sudoku with 1 row with numbers 1 to 9.
*/


#ifndef GENERATE_H_INCLUDED
#define GENERATE_H_INCLUDED
#define UNASSIGNED 0
#include<iostream>
#include<stdlib.h>
#include<ctime>

using namespace std;

class Sudoku
{
    public:
    int puzzle[9][9];
    int srow;
    int scol;
    bool SolveSudoku(Sudoku &gird,Sudoku &grid2,int c,int &count);
    int generate(Sudoku &grid1,Sudoku &grid2,int level);//
    int print_array(Sudoku &grid);//
    int generate_final(Sudoku &grid1,Sudoku &grid2,int level);
    bool check_equal(Sudoku &grid1,Sudoku &grid2);//
    bool check_unique(Sudoku &grid1,Sudoku &grid2);
    int equal_array(Sudoku &grid1,Sudoku &grid2);//
    bool FindUnassignedPos(Sudoku &grid,int &r,int &c);//
    bool NoConflict(Sudoku &grid,int r,int c,int num);//
};

/*The functiom FindUnassignedPos is a public member of the Sudoku class.
  It has a return type of bool it takes the object we are solving starts from the
  given starting point and finds the first unassigned location and returns true if
  found. As the row and coulunm are passed by referance this function also sets the row and coloumn to that
  location.*/

bool Sudoku::FindUnassignedPos(Sudoku &grid,int &r,int &c)
{
    int i,j;
    int count=0;
    int k=grid.scol;
    for(i=grid.srow;i<9;i++)
    {
        if(count>0)
        {
            k=0;
        }
        for(j=k;j<9;j++)
        {
            if(grid.puzzle[i][j]==UNASSIGNED)
            {
                r=i;
                c=j;
                return true;
            }
            count++;
            if(count==81)
            {
                return false;
            }
        }
        if(i==8&&j==9)
        {
            i=-1;
            j=0;
        }
    }
}

/*The funcion NoConflict takes the object of Grid and the row coloumn and checks
  if the given number comlplies with the given tules of sudoku at thet position.*/

bool Sudoku::NoConflict(Sudoku &grid,int r,int c,int num)
{
    int i,j,k,l;

    for(i=0;i<9;i++)
    {
        if(grid.puzzle[r][i]==num||grid.puzzle[i][c]==num)
        {
            return false;
        }
    }
    for(i=0;i<=6;i+=3)
    {
        for(j=0;j<=6;j+=3)
        {
            if(r<i+3&&r>=i&&c<j+3&&c>=j)
            {
                for(k=i;k<i+3;k++)
                {
                    for(l=j;l<j+3;l++)
                    {
                        if(grid.puzzle[k][l]==num)
                        {
                            return false;
                        }
                    }
                }
            }
        }
    }
    return true;
}


/*This is the function that is called to solve the sudoku it
  uses a backtracing algorithm to solve the sudoku. If the parameter c is zero then the function
  checks numbers from 1 to 9 and if it is any thing else it checks from 9 to 1. The Sudoku that is solved is the first
  parameter adn if someone wants to check if the solution is equal to any other sudoku then they pass the sudoku they want to
  compare with as the second parameter. If the solution is equal to the sudoku then the varialbe count is set to 1.*/

bool Sudoku::SolveSudoku(Sudoku &grid,Sudoku &grid1,int c,int &count)
{
    int row,col;

    if(!grid.FindUnassignedPos(grid,row,col))
    {
        if(check_equal(grid,grid1))
        {
            count=1;
        }
        return true;
    }


    if(c==0)
    {
        for(int i=1;i<=9;i++)
        {
            if(grid.NoConflict(grid,row,col,i))
            {
                grid.puzzle[row][col]=i;
                //system("cls");
                //grid.print_array(grid);
                if(grid.SolveSudoku(grid,grid1,c,count))
                {
                    return true;
                }
                grid.puzzle[row][col]=UNASSIGNED;
            }
        }
    }
    else
    {
        for(int i=9;i>=1;i--)
        {
            if(grid.NoConflict(grid,row,col,i))
            {
                grid.puzzle[row][col]=i;
                //system("cls");
                //grid.print_array(grid);
                if(grid.SolveSudoku(grid,grid1,c,count))
                {
                    return true;
                }
                grid.puzzle[row][col]=UNASSIGNED;
            }
        }
    }

    return false;
}

/*This is the main function that is  called it requires two parameters of the Sudoku type the first one
  will later contain the complete solved sudoku  and the second one will contain a sudoku with numbers removed equal to the third
  parameter.*/
int Sudoku::generate(Sudoku &grid1,Sudoku &grid2,int level)
{
    int i,j,num;
    int dummy;

    for(i=0;i<9;i++)
    {
        for(j=0;j<9;j++)
        {
            grid1.puzzle[i][j]=UNASSIGNED;
            grid2.puzzle[i][j]=UNASSIGNED;
        }
    }

    i=0;
    srand(time(0));
    j=0;

    while(i<9)
    {
        num=rand()%9+1;
        if(grid1.NoConflict(grid1,j,i,num))
        {
            grid1.puzzle[j][i]=num;
            i++;
        }
    }

    grid1.srow=rand()%3;
    grid1.scol=rand()%9;
    grid1.SolveSudoku(grid1,grid1,0,dummy);
    grid2.generate_final(grid1,grid2,level);
    return 0;
}

/*The following function prints the object passed. This has no relevence to the actual generation
  but is used during testing and can be used to check it the sudokus are generating correctly
  without graphics.*/
int Sudoku::print_array(Sudoku &grid)
{
    for(int i=0;i<9;i++)
    {
        for(int j=0;j<9;j++)
        {
            cout<<grid.puzzle[i][j]<<" ";
        }
        cout<<endl;
    }
    return 0;
}

/*the following function takes two objects of the sudoku class and copies the array of the second oject to that of the first.*/
int Sudoku::equal_array(Sudoku &grid1,Sudoku &grid2)
{
    for(int i=0;i<9;i++)
    {
        for(int j=0;j<9;j++)
        {
            grid1.puzzle[i][j]=grid2.puzzle[i][j];
        }
    }
    return 0;
}

/*The following function takes two objects of the type Sudoku and checks if their member arrays are equal to each
   other.*/

bool Sudoku::check_equal(Sudoku &grid1,Sudoku &grid2)
{
    for(int i=0;i<9;i++)
    {
        for(int j=0;j<9;j++)
        {
            if(grid1.puzzle[i][j]!=grid2.puzzle[i][j])
            {
                return false;
            }
        }
    }
    return true;
}

/*The following function takes the same input as the generate function but it is required that you pass a solved/completed sudoku
  as the first parameter and a sudoku in which you require "level" number of numbers form random positions.*/
int Sudoku::generate_final(Sudoku &grid1,Sudoku &grid2,int level)
{
    int Pos[2][81];
    int i,j,k=0,temp;

    srand(time(0));

    for(i=0;i<9;i++)
    {
        for(j=i*9;j<i*9+9;j++)
        {
            Pos[0][j]=i;
            Pos[1][j]=k++;
        }
        k=0;
    }
    for(i=0;i<80;i++)
    {
        int r=i+(rand()%(81-i));
        int temp = Pos[0][i]; Pos[0][i] = Pos[0][r]; Pos[0][r] = temp;
        temp = Pos[1][i]; Pos[1][i] = Pos[1][r]; Pos[1][r] = temp;
    }
    i=0;
    k=0;
    j=0;

    equal_array(grid2,grid1);
    while(i<level)
    {
        if(k==81)
        {
            break;
        }

        temp=grid2.puzzle[Pos[0][k]][Pos[1][k]];
        grid2.puzzle[Pos[0][k]][Pos[1][k]]=UNASSIGNED;

        if(grid2.check_unique(grid1,grid2))
        {
            i++;
            k++;
            continue;
        }

        grid2.puzzle[Pos[0][k]][Pos[1][k]]=temp;
        k++;
    }
    return 0;
}

/*The following funtion takes a sudoku with numbers removed and checks if it has a unique solution.
  It solves it with the c parameter of the function SolveSudoku set to one and then to 0 if in both cases count is 1
  then the sudolu has a unique solution.*/
bool Sudoku::check_unique(Sudoku &grid1,Sudoku &grid2)
{
    int i=0,k=0;
    Sudoku grid;
    int count1=0,count2=0;

    grid.equal_array(grid,grid2);
    grid.scol=rand()%9;
    grid.srow=rand()%9;
    grid.SolveSudoku(grid,grid1,1,count1);
    grid.equal_array(grid,grid2);
    grid.scol=rand()%9;
    grid.srow=rand()%9;
    grid.SolveSudoku(grid,grid1,1,count2);
    if(count1==1&&count2==1)
    {
        return true;
    }
    return false;

}


#endif // GENERATE_H_INCLUDED
